<?php
/*
 * This file contains the plugins for table- and tab-style submenus.
 */


class _dqx_adminmenu_MenuTree {
  protected $_submenus;
  protected $_items;
  protected $_inline;

  function __construct(array $submenus, array $items, array $inline_children) {
    $this->_submenus = $submenus;
    $this->_items = $items;
    $this->_inline = $inline_children;
  }

  function renderMenu($parent_path) {
    return $this->_renderSubmenu($parent_path, $ul_attr);
  }

  function pathAccess($path) {
    $item = $this->_items[$path];
    if ($item && isset($item['access'])) {
      return $item['access'];
    }
    else {
      $router_item = menu_get_item($path);
      return $router_item['access'];
    }
  }

  function getItem($key) {
    return isset($this->_items[$key]) ? $this->_items[$key] : NULL;
  }

  function getSubmenu($key) {
    return isset($this->_submenus[$key]) ? $this->_submenus[$key] : NULL;
  }

  protected function _renderSubmenu($parent_path) {
    $html = '';
    $submenu = $this->_submenus[$parent_path];
    if (is_object($submenu)) {
      return $submenu->render($this);
    }
    else if (is_array($submenu)) {

      if (isset($this->_inline[$parent_path])) {
        $inline_path = $this->_inline[$parent_path];
        unset($submenu[$inline_path]);
        if (isset($this->_items[$inline_path])) {
          $inline_item = &$this->_items[$inline_path];
          $inline_item['submenu_attributes']['class'] .= ' dqx_adminmenu-inline';
          $inline_item['item_attributes']['class'] .= ' dqx_adminmenu-inline';
          $inline_item['localized_options']['attributes']['class'] .= ' dqx_adminmenu-inline';
          $inline_html = $this->_renderSubmenuItem($inline_path, $inline_item);
        }
      }

      $items_html = $this->_renderSubmenuItems($submenu);
      $items_html .= $inline_html;
    }
    $ul_attr = dqx_adminmenu_extract_attributes($this->_items[$parent_path], 'submenu_attributes');
    return $ul_attr->UL($items_html);
  }

  protected function _renderSubmenuItems($submenu) {
    $sort = array();
    $pieces = array();
    foreach ($submenu as $k => $path) {
      if ($item = $this->_items[$path]) {
        $item_html = $this->_renderSubmenuItem($path, $item);
        if (strlen($item_html)) {
          $pieces[$k] = $item_html;
          $sort[$k] = (50000 + $item['weight']) . ' ' . $item['title'];
        }
      }
    }
    array_multisort($sort, $pieces);
    return implode('', $pieces);
  }

  protected function _renderSubmenuItem($path, $item) {

    // late access check
    if (!isset($item['access'])) {
      $router_item = menu_get_item($item['link_path']);
      $item['access'] = $router_item['access'];
    }

    $subtree_html = $this->_renderSubmenu($path);

    $options = $item['localized_options'];
    $options = is_array($options) ? $options : array();

    $link_attr = dqx_adminmenu_link_attributes($item['link_path'], $options);
    $li_attr = dqx_adminmenu_extract_attributes($item, 'item_attributes');

    if (!$item['access']) {
      if (!$subtree_html) {
        return;
      }
      // no access, but still want to reveal some of the subtree items
      $li_attr->addClass('no-access');
      $link_attr->addClass('no-access');
      $link_attr->removeAttribute('href');
    }

    if ($subtree_html) {
      $li_attr->addClass('expandable');
      $link_attr->addClass('expandable');
    }

    $link_html = $link_attr->A($item['title'], $options['html']);
    return $li_attr->LI($link_html . $subtree_html);
  }
}


function dqx_adminmenu_link_attributes($path, array $options) {
  $obj = dqx_adminmenu_extract_attributes($options, 'attributes', FALSE);
  $obj->setAttribute('href', url($path, $options));
  return $obj;
}


function dqx_adminmenu_extract_attributes(array $info_array, $array_key, $raw = TRUE) {
  $attr = array();
  if (is_array($info_array[$array_key])) {
    $attr = $info_array[$array_key];
  }
  return dqx_adminmenu_tag_attributes($attr, $raw);
}


function dqx_adminmenu_tag_attributes($attr = NULL, $raw = TRUE) {
  if (is_object($attr)) {
    return $attr;
  }
  else {
    $obj = new dqx_adminmenu_HtmlTagAttributes();
    if (is_array($attr)) {
      $obj->setAttributes($attr, $raw);
      if (isset($attr['class'])) {
        $obj->addClasses($attr['class']);
      }
    }
    return $obj;
  }
}


function dqx_adminmenu_class_attribute($classes) {
  if (!is_object($classes)) {
    $attr = new dqx_adminmenu_HtmlTagAttributes();
    $attr->addClasses($classes);
    return $attr;
  }
  else {
    // assume that it is already a TagAttributes object
    return $classes;
  }
}


class dqx_adminmenu_HtmlTagAttributes {
  protected $_attributes = array();
  protected $_classes = array();

  function setAttribute($key, $value, $raw = TRUE) {
    $this->_attributes[$key] = $raw ? $value : check_plain($value);
  }

  function unsetAttribute($key) {
    unset($this->_attributes[$key]);
  }

  function setAttributes(array $values, $raw = TRUE) {
    foreach ($values as $key => $value) {
      $this->_attributes[$key] = $raw ? $value : check_plain($value);
    }
  }

  function addClass($class) {
    $this->_classes[$class] = $class;
  }

  function addClassIf($cond, $class) {
    if ($cond) {
      $this->_classes[$class] = $class;
    }
  }

  function addClasses($classes) {
    if (!is_array($classes)) {
      $classes = explode(' ', $classes);
    }
    foreach ($classes as $class) {
      if ($class) {
        $this->_classes[$class] = $class;
      }
    }
  }

  function addClassesIf(array $classes_if) {
    foreach ($classes_if as $class => $cond) {
      if ($cond) {
        $this->_classes[$class] = $class;
      }
    }
  }

  function hasClass($class) {
    return isset($this->_classes[$class]);
  }

  function removeClass($class) {
    unset($this->_classes[$class]);
  }

  function getAttributes() {
    if (!empty($this->_classes)) {
      $this->_attributes['class'] = implode(' ', $this->_classes);
    }
    else {
      unset($this->_attributes['class']);
    }
    return $this->_attributes;
  }

  function __toString() {
    $t = '';
    foreach ($this->getAttributes() as $key => $value) {
      $t .= ' '. $key .'="'. $value .'"';
    }
    return $t;
  }

  function renderTag($tagname, $content = NULL, $raw = TRUE) {
    if (isset($content)) {
      if (!$raw) {
        $content = check_plain($content);
      }
      return '<'. $tagname . $this->__toString() .'>'. $content .'</'. $tagname .">\n";
    }
    else {
      return '<'. $tagname . $this->__toString() .' />';
    }
  }

  function __call($tagname, $args) {
    if (preg_match('/^[A-Z]+$/', $tagname)) {
      list($content, $raw, $empty_text) = $args + array(NULL, TRUE, '');
      $tagname = strtolower($tagname);
      if (!strlen($content) && is_string($empty_text)) {
        return $empty_text;
      }
      else {
        return $this->renderTag($tagname, $content, $raw);
      }
    }
  }
}





